#include <stdio.h>
#include <math.h>
#include <sys/time.h>

void printf();

#ifdef TESTING
static int numoftrials = 1; /*300*/;
#else
static int numoftrials = 100; /*300*/;
#endif

#ifdef TESTING
#define PRECISION 4
#else
#define PRECISION 256
#endif
#define MULT_TABLE_MAX 16  /*if > 16, must change table*/
#define COARSENESS 0x8000
#define COARSENESS_IN_BITS 16
typedef struct {
  int lower;
  int upper;
} tempevent_type;

unsigned char mult_table[MULT_TABLE_MAX][MULT_TABLE_MAX];


typedef struct {
  int left[PRECISION+2];
  int right[PRECISION+2];
} tempdist_type;

typedef enum {FALSE, TRUE} bool;
typedef enum {LEFT_RIGHT, RIGHT_LEFT} direction_type;

tempdist_type uniform;
unsigned int uniform_tree_height = 0;

int maxmaxqueue = 0;
int maxqueue = 0;

int pregammas[101];

typedef struct {
  int height;
  bool active;
  unsigned char leftrods;
  unsigned char rightrods;
  void *left;
  void *right;
} tree_node_type;

tree_node_type spare_nodes[1000];
int allocated = 0;
tree_node_type *uniform_tree;

typedef struct {
  unsigned char round;
  unsigned int agranuleslower;
  unsigned int agranulesupper;
  unsigned int bgranuleslower;
  unsigned int bgranulesupper;
  tree_node_type *arodstree;
  tree_node_type *brodstree;
} queue_node_type;

queue_node_type queue[2000];

tree_node_type *build_uniform_tree(int current,int height) {
  tree_node_type *node;

  node = &spare_nodes[allocated++];
  if (current <= 1) {
    node->active = FALSE;
    node->left = node;
    node->right = node;
    node->leftrods = 0;
    node->rightrods = 0;
    }
  else {
    node->active = TRUE;
    node->left = (void *)build_uniform_tree(current >> 1,height+1);
    node->right = (void *)build_uniform_tree(current >> 1,height+1);
    node->leftrods = current >> 1;
    node->rightrods = current >> 1;
    }
  if (height > uniform_tree_height) uniform_tree_height = height;
  node->height = height;
  return node;
}

void main(argc, argv) 
  int argc;
  char *argv[]; {
  tree_node_type *build_uniform_tree();
  int i, j, k, sum;
  int times[101];
  int temps[101];
  void gettimeofday();
  bool before();
  struct timeval tp1, tp2;
  int temp1, temp2;
  int gamma, temp;
  tempevent_type alpha, beta;
  double f;

  temp = 0;
  alpha.lower = 1;
  alpha.upper = 30;
  beta.lower = 1;
  beta.upper = 30; 
  if (argc > 3) {
    alpha.lower = atoi(argv[1]);
    alpha.upper = atoi(argv[2]);
    beta.lower = atoi(argv[3]);
    beta.upper = atoi(argv[4]);
    }

  for (i=0;i<MULT_TABLE_MAX;i++) for (j=0;j<MULT_TABLE_MAX;j++) mult_table[i][j] = i*j;

#define GAMMA_MULT PRECISION * PRECISION / 100.0
  for (i=0;i<101;i++) {
    f = i;
    pregammas[i] = (int) ceil(f * GAMMA_MULT);
    /* printf("%d %d %f \n", i, pregammas[i], (i * (GAMMA_MULT)));*/
    }

  uniform_tree = build_uniform_tree(PRECISION,0);

  for (i=1+numoftrials;i<=100+numoftrials;i++) {

    gettimeofday(&tp1,NULL);

    for (j=0; j<numoftrials; j++) temp += before(i - numoftrials,&alpha,&beta);

    gettimeofday(&tp2,NULL);

    temp1 = tp1.tv_sec * 1000000 + tp1.tv_usec;
    temp2 = tp2.tv_sec * 1000000 + tp2.tv_usec;
    temp1 = temp2 - temp1;

    times[i-numoftrials] = temp1;
    temps[i-numoftrials] = temp;
    }
    for (sum=0,i = 1; i <= 100; i++) {
      sum += times[i];
#ifdef GRAPHING
      printf("%d\n", times[i]/numoftrials);
#else
      printf("%d %d %d\n", temps[i], i, times[i]/numoftrials);
#endif
      }

#ifndef GRAPHING
   printf("Average %d\n", sum/numoftrials);
   printf("m %d\n", maxmaxqueue);
#endif
}

#ifdef JUNKTESTING
    if (((agl) >> 16) < (((agu) - 1) >> 16) || 
       ((bgl) >> 16) < (((bgu) - 1) >> 16)) {
#endif

#define PUSH(agl,agu,ar,bgl,bgu,br,r) \
  if ((((tree_node_type *) ar)->active) && (((tree_node_type *) br)->active)) {\
    if (r < COARSENESS_IN_BITS) {\
    queue_ptr = &queue[maxqueue++]; \
    queue_ptr->agranuleslower = agl; \
    queue_ptr->agranulesupper = agu; \
    queue_ptr->arodstree = ar; \
    queue_ptr->bgranuleslower = bgl; \
    queue_ptr->bgranulesupper = bgu; \
    queue_ptr->brodstree = br; \
    queue_ptr->round = r; \
    }}

#define POP(agl,agu,ar,bgl,bgu,br,r) \
 if (currentqueue == maxqueue) return FALSE; \
 else { \
  queue_ptr = &queue[currentqueue++]; \
  agl = queue_ptr->agranuleslower; \
  agu = queue_ptr->agranulesupper; \
  ar = queue_ptr->arodstree; \
  bgl = queue_ptr->bgranuleslower; \
  bgu = queue_ptr->bgranulesupper; \
  br = queue_ptr->brodstree; \
  r = queue_ptr->round; \
  }

bool before(int gamma, tempevent_type *alpha, tempevent_type *beta) 
{
  int  target, mal_target, i, j, round, foundround, roundt;
  tree_node_type *aleft, *aright, *bleft, *bright, *bprevious, *aprevious;
  unsigned int agranulesmidapprox, bgranulesmidapprox, currentqueue /*, maxqueue*/;
  tree_node_type *arodstree, *brodstree, *foundarodstree, *foundbrodstree, 
                 *atrodstree, *btrodstree;
  direction_type direction;
  queue_node_type *queue_ptr;
  unsigned int agranulesmid, bgranulesmid, arodsleft, brodsleft,
	       brodsright, arodsright, agranuleslower, agranulesupper,
               bgranuleslower, bgranulesupper, btgranulesmid, atgranulesmid,
               btgranulesmidapproxceil, atgranulesmidapproxceil, 
               btgranulesmidapproxfloor, atgranulesmidapproxfloor,
               bgranulesmidapproxceil, agranulesmidapproxceil, 
               bgranulesmidapproxfloor, agranulesmidapproxfloor,
               agranulesrange, bgranulesrange, 
               foundagranulesrange, foundbgranulesrange,
               foundagranulesmid, foundbgranulesmid,
	       accumaleft, accumbright, accumbleft, accumaright, 
	       atgranulesrange, btgranulesrange;

#define MULT(i,j) ((i < MULT_TABLE_MAX) && (j < MULT_TABLE_MAX))? mult_table[i][j]: i*j

  /* Determinate precedes? */
  if (beta->upper < alpha->lower) return FALSE;
  if (alpha->upper < beta->lower) return TRUE;

  /* Assert --> The periods of indeterminacy must overlap */

  /* Need a determinate before?*/ /* Is this true? */
  if (gamma == 100) return FALSE;

  /* Is the indeterminacy trivial? */ /* Is this true? */
  if (gamma == 1) return TRUE;

  target = pregammas[gamma];
  mal_target = pregammas[100 - gamma];

  agranuleslower = alpha->lower << 16;
  agranulesupper = (alpha->upper + 1) << 16;
  arodstree = uniform_tree; 
  bgranuleslower = beta->lower << 16;
  bgranulesupper = (beta->upper + 1) << 16;
  brodstree = uniform_tree; 

  maxmaxqueue = (maxqueue > maxmaxqueue)? maxqueue: maxmaxqueue;
  currentqueue = 0;
  maxqueue = 0;
  round = 1;

  do {

    /* ALPHA */
    /* Determine midpoint granule */
    agranulesrange = (agranulesupper - agranuleslower) >> 1;
    agranulesmid = agranulesrange + agranuleslower;
    agranulesmidapproxfloor = (agranulesmid - 1) >> 16; 
    agranulesmidapproxceil = agranulesmid >> 16; 
  
    /* BETA */
    /* Determine midpoint granule */
    bgranulesrange = (bgranulesupper - bgranuleslower) >> 1;
    bgranulesmid = bgranulesrange + bgranuleslower;
    bgranulesmidapproxfloor = (bgranulesmid - 1) >> 16; 
    bgranulesmidapproxceil = bgranulesmid >> 16; 

  if (agranulesmidapproxfloor < bgranulesmidapproxceil) {

    accumaleft = 0;
    accumbright = 0;
    atgranulesmid = agranulesmid;
    btgranulesmid = bgranulesmid;
    atgranulesrange = agranulesrange;
    btgranulesrange = bgranulesrange;
    do {
      atgranulesrange = atgranulesrange >> 1;
      atgranulesmid += atgranulesrange;
      atgranulesmidapproxfloor = (atgranulesmid - 1) >> 16; 
      btgranulesrange = btgranulesrange >> 1;
      btgranulesmid -= btgranulesrange;
      btgranulesmidapproxceil = btgranulesmid >> 16; 
      if (atgranulesmidapproxfloor >= btgranulesmidapproxceil) break;
      accumaleft += arodstree->leftrods;
      accumbright += brodstree->rightrods;
      arodstree = arodstree->right;
      brodstree = brodstree->left;
      if (!brodstree->active && !arodstree->active) break;
      round++;
      agranulesmid = atgranulesmid;
      bgranulesmid = btgranulesmid;
      agranulesrange = atgranulesrange;
      bgranulesrange = btgranulesrange;
    } while (round < COARSENESS_IN_BITS);

    /* left, right */
#ifdef TESTING
    printf("ALPHA BEFORE BETA\n");
    printf("gamma %d arodstree->leftrods %d accumaleft %d\n",
       gamma,arodstree->leftrods,accumaleft);
    printf("target %d brodstree->rightrods %d accumbright %d\n",
       target,brodstree->rightrods,accumbright);
#endif
    arodsleft = arodstree->leftrods + accumaleft;
    brodsright = brodstree->rightrods + accumbright;
    target -= MULT(arodsleft,brodsright);
    if (target <= 0) return TRUE;

    /* left, left*/
    PUSH(agranulesmid - agranulesrange,agranulesmid,arodstree->left,
         bgranulesmid - bgranulesrange,bgranulesmid,brodstree->left,
         round);

    /* right, left*/
    PUSH(agranulesmid,agranulesmid + agranulesrange,arodstree->right,
         bgranulesmid - bgranulesrange,bgranulesmid,brodstree->left,
         round);

    /* right, right*/
    PUSH(agranulesmid,agranulesmid + agranulesrange,arodstree->right,
         bgranulesmid,bgranulesmid + bgranulesrange,brodstree->right,
         round);
    }

  else if (bgranulesmidapproxfloor < agranulesmidapproxceil) {

    accumbleft = 0;
    accumaright = 0;
    atgranulesmid = agranulesmid;
    btgranulesmid = bgranulesmid;
    atgranulesrange = agranulesrange;
    btgranulesrange = bgranulesrange;
    do {
      atgranulesrange = atgranulesrange >> 1;
      atgranulesmid -= atgranulesrange;
      atgranulesmidapproxceil = atgranulesmid >> 16; 
      btgranulesrange = btgranulesrange >> 1;
      btgranulesmid += btgranulesrange;
      btgranulesmidapproxfloor = (btgranulesmid - 1) >> 16; 
      if (btgranulesmidapproxfloor >= atgranulesmidapproxceil) break;
      accumbleft += brodstree->leftrods;
      accumaright += arodstree->rightrods;
      arodstree = arodstree->left;
      brodstree = brodstree->right;
      if (!brodstree->active && !arodstree->active) break;
      round++;
      agranulesmid = atgranulesmid;
      bgranulesmid = btgranulesmid;
      agranulesrange = atgranulesrange;
      bgranulesrange = btgranulesrange;
    } while (round < COARSENESS_IN_BITS);

    /* right, left*/
#ifdef TESTING
    printf("BETA BEFORE ALPHA\n");
    printf("gamma %d arodstree->rightrods %d accumaright %d\n",
       gamma,arodstree->rightrods,accumaright);
    printf("target %d brodstree->leftrods %d accumbleft %d\n",
       target,brodstree->leftrods,accumbleft);
#endif
    arodsright = arodstree->rightrods + accumaright;
    brodsleft = brodstree->leftrods + accumbleft;
    mal_target -= MULT(arodsright,brodsleft);
    if (mal_target <= 0) return FALSE;

    /* left, right */
    PUSH(agranulesmid - agranulesrange,agranulesmid,arodstree->left,
         bgranulesmid,bgranulesmid + bgranulesrange,brodstree->right,
         round);

    /* left, left*/
    PUSH(agranulesmid - agranulesrange,agranulesmid,arodstree->left,
         bgranulesmid - bgranulesrange,bgranulesmid,brodstree->left,
         round);

    /* right, right*/
    PUSH(agranulesmid,agranulesmid + agranulesrange,arodstree->right,
         bgranulesmid,bgranulesmid + bgranulesrange,brodstree->right,
         round);
    }

  else {

   direction = LEFT_RIGHT;
   foundround = 0;
   atgranulesmid = agranulesmid;
   btgranulesmid = bgranulesmid;
   atgranulesrange = agranulesrange;
   btgranulesrange = bgranulesrange;
   atrodstree = arodstree;
   btrodstree = brodstree;
   roundt = round;
   accumaleft = 0;
   accumbright = 0;
   do {
     if (direction == LEFT_RIGHT) { 
       atrodstree = atrodstree->left;
       btrodstree = btrodstree->right;
       atgranulesrange = atgranulesrange >> 1;
       atgranulesmid -= atgranulesrange;
       atgranulesmidapproxfloor = (atgranulesmid - 1) >> 16; 
       btgranulesrange = btgranulesrange >> 1;
       btgranulesmid += btgranulesrange;
       btgranulesmidapproxceil = btgranulesmid >> 16; 
       if (atgranulesmidapproxfloor < btgranulesmidapproxceil) {
#ifdef TESTING
         printf("LEFT_RIGHT round %d agmid %d bgmid %d\n",
	   roundt, atgranulesmid, btgranulesmid);
#endif
         direction = RIGHT_LEFT;
         foundarodstree = atrodstree;
         foundbrodstree = btrodstree;
         accumaleft += foundarodstree->leftrods; 
         accumbright += foundbrodstree->rightrods; 
         foundagranulesmid = atgranulesmid;
         foundbgranulesmid = btgranulesmid;
         foundround = roundt;
         }
       else { /* do nothing */
         }
       }
     else { /*  for RIGHT_LEFT */
       atrodstree = atrodstree->right;
       btrodstree = btrodstree->left;
       atgranulesrange = atgranulesrange >> 1;
       atgranulesmid += atgranulesrange;
       atgranulesmidapproxceil = atgranulesmid >> 16; 
       btgranulesrange = btgranulesrange >> 1;
       btgranulesmid -= btgranulesrange;
       btgranulesmidapproxfloor = (btgranulesmid - 1) >> 16; 
       if (atgranulesmidapproxceil < btgranulesmidapproxfloor) {
         foundarodstree = atrodstree;
         foundbrodstree = btrodstree;
#ifdef TESTING
         printf("RIGHT_LEFT round %d agmid %d bgmid %d tree %d\n",
	   roundt, atgranulesmid, btgranulesmid, foundarodstree->leftrods);
#endif
         accumaleft += foundarodstree->leftrods; 
         accumbright += foundbrodstree->rightrods; 
         foundagranulesmid = atgranulesmid;
         foundbgranulesmid = btgranulesmid;
         foundround = roundt;
         }
       else { 
         direction = LEFT_RIGHT;
         }
       }
       roundt++;
       if (!btrodstree->active && !atrodstree->active) break;
     } while (roundt < COARSENESS_IN_BITS);

    /* one such place found */
    if (foundround) {

      /* left, right */
#ifdef TESTING
    printf("ALPHA EQ BETA\n");
    printf("gamma %d arodstree->leftrods %d accumaleft %d\n",
       gamma,arodstree->leftrods,accumaleft);
    printf("target %d brodstree->rightrods %d accumbright %d\n",
       target,brodstree->rightrods,accumbright);
#endif
      arodsleft = foundarodstree->leftrods + accumaleft;
      brodsright = foundbrodstree->rightrods + accumbright;
      target -= MULT(arodsleft,brodsright);
      if (target <= 0) return TRUE;

      /* right, left */
      /* left, left*/
      /* right, right*/
      /* in same granule, do nothing */
      }
   
    /* right, left */
    arodsright = arodstree->rightrods;
    brodsleft = brodstree->leftrods;
    mal_target -= MULT(arodsright,brodsleft);
    if (mal_target <= 0) return FALSE;

    /* left, left */
    PUSH(agranuleslower,agranulesmid,arodstree->left,
         bgranuleslower,bgranulesmid,brodstree->left,round);

    /* right, right */
    PUSH(agranulesmid,agranulesupper,arodstree->right,
         bgranulesmid,bgranulesupper,brodstree->right,round);

    }

  POP(agranuleslower,agranulesupper,arodstree,
      bgranuleslower,bgranulesupper,brodstree,round);


  } while (TRUE);
  /* should never get here */
  return FALSE;
}
