/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/calendar/gregorian/gregorian_comparison.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef gregorian_comparision
#define gregorian_comparision
/* comparison.h
*
*  This module contains all procedures for variable
*  span comparison.
*
*  Procedures in this file are:
*
*  vs_eq_fs
*  vs_eq_vs
*  vs_lt_fs
*  fs_lt_vs
*  vs_lt_vs
*  fs_gt_vs
*  vs_gt_fs
*  vs_gt_vs
*
*  where fs = fixed span, and vs = variable span
*/

/*
*  Routine:  cal_standard_gregorian_v1_vs_eq_fs
*
*  Description:  Compare a variable span to a fixed span.  Returns 
*	 	 the boolean value TRUE if they are equal and
*                FALSE otherwise.
*
*  Arguments:   vs     -- (IN) : variable span
*  		fs     -- (IN) : fixed span
*
*  Return Value: bool : TRUE if vs = fs
*		        FALSE if vs <> fs
*
*  Errors: semantic_error
*
*  Side Effects: None
*/

bool cal_standard_gregorian_v1_vs_eq_fs(span_type vs, span_type fs);

/*
*  Routine:  cal_standard_gregorian_v1_vs_eq_vs
*
*  Description:  Compare a variable span to a variable span.  Returns 
*	 	 the boolean value TRUE if they are equal and
*                FALSE otherwise.
*
*  Arguments:   vs1    -- (IN) : variable span
*  		vs2    -- (IN) : variable span
*
*  Return Value: bool : TRUE if vs1 = vs2
*                       FALSE if vs1 <> vs2
*
*  Errors: semantic_error
*
*  Side Effects: None
*/

bool cal_standard_gregorian_v1_vs_eq_vs(span_type vs1, span_type vs2);

/*
*  Routine:  cal_standard_gregorian_v1_vs_lt_fs
*
*  Description:  Compare a variable span to a fixed span.  Returns 
*               the boolean value TRUE if the first span is less than
*               the second span and FALSE otherwise.
*
*  Arguments:   vs     -- (IN) : variable span
*  		fs     -- (IN) : fixed span
*
*  Return Value: bool: TRUE if vs < fs
*		       FALSE if vs >= fs
*
*  Errors: semantic_error
*
*  Side Effects: None
*/

bool cal_standard_gregorian_v1_vs_lt_fs(span_type vs, span_type fs);

/*
*  Routine:  cal_standard_gregorian_v1_fs_lt_vs
*
*  Description:  Compare a fixed span to a variable span.  Returns 
*               the boolean value TRUE if the first span is less than
*               the second span and FALSE otherwise.
*
*  Arguments:   fs     -- (IN) : fixed span
*  		vs     -- (IN) : variable span
*
*  Return Value: bool : TRUE if fs < vs
*                       FALSE if fs >= vs
*
*  Errors: semantic_error
*
*  Side Effects: None
*/

bool cal_standard_gregorian_v1_fs_lt_vs(span_type fs, span_type vs);

/*
*  Routine:  cal_standard_gregorian_v1_vs_lt_vs
*
*  Description:  Compare a variable span to a variable span.  Returns 
*               the boolean value TRUE if the first span is less than
*               the second span and FALSE otherwise.
*
*  Arguments:   vs1    -- (IN) : variable span
*  		vs2    -- (IN) : variable span
*
*  Return Value: bool : TRUE if vs1 < vs2
*		        FALSE if vs1 >= vs2
*
*  Errors: semantic_error
*
*  Side Effects: None
*/

bool cal_standard_gregorian_v1_vs_lt_vs(span_type vs1, span_type vs2);

/*
*  Routine:  cal_standard_gregorian_v1_fs_gt_vs
*
*  Description:  Compare a fixed span to a variable span.  Returns 
*               the boolean value TRUE if the first span is greater than
*               the second span and FALSE otherwise.
*
*  Arguments:   fs     -- (IN) : fixed span
*  		vs     -- (IN) : variable span
*
*  Return Value: bool: TRUE if fs > vs
*		       FALSE if fs <= vs
*
*  Errors: semantic_error
*
*  Side Effects: None
*/

bool cal_standard_gregorian_v1_fs_gt_vs(span_type fs, span_type vs);

/*
*  Routine:  cal_standard_gregorian_v1_vs_gt_fs
*
*  Description:  Compare a variable span to a fixed span.  Returns 
*               the boolean value TRUE if the first span is greater than
*               the second span and FALSE otherwise.

*  Arguments:   vs     -- (IN) : variable span
*  		fs     -- (IN) : fixed span
*
*  Return Value: bool : TRUE if vs > fs
*                       FALSE if vs <= fs
*
*  Errors: semantic_error
*
*  Side Effects: None
*
*/

bool cal_standard_gregorian_v1_vs_gt_fs(span_type vs, span_type fs);

/*
*  Routine:  cal_standard_gregorian_v1_vs_gt_vs
*
*  Description:  Compare a variable span to a variable span.  Returns 
*               the boolean value TRUE if the first span is greater than
*               the second span and FALSE otherwise.
*
*  Arguments:   vs1    -- (IN) : variable span
*  		vs2    -- (IN) : variable span
*
*  Return Value: bool :  TRUE if vs1 < vs2
*		         FALSE if vs1 >= vs2
*
*  Return Value: Error code
*
*  Errors: semantic_error
*
*  Side Effects: None
*
*/

bool cal_standard_gregorian_v1_vs_gt_vs(span_type vs1, span_type vs2);
#endif
