/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/calendar/gregorian/gregorian_input.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef gregorian_input
#define gregorian_input

#include "ucs/value_array.h"
/* input.h
*
*  This module contains all function supporting temporal 
*  constant to time-stamp translation.
*
*/

/*
*  Routine:  cal_standard_gregorian_v1_gen_s_timestamp
*
*  Description:  Create a span time-stamp from a parsed version of the 
*		 input string
*
*  Arguments: val_array -- (IN) : parsed representation of the input string
*             span      -- (OUT): span time-stamp
*
*  Return Value: Error code
*
*  Errors: conversion_error
*
*  Side Effects: Overwrites span
*/

error_type cal_standard_gregorian_v1_gen_s_timestamp(value_array_type *val_array, span_type span);

/*
*  Routine:  cal_standard_gregorian_v1_gen_e_timestamp
*
*  Description:  Create an event time-stamp from a parsed version of the 
*		 input string
*
*  Arguments: val_array  -- (IN) : parsed representation of the input string
*             locale     -- (IN) : current locale property value
*             event      -- (OUT): event time-stamp
*
*  Return Value: Error code
*
*  Errors: conversion_error
*
*  Side Effects: Overwrites event
*/

error_type cal_standard_gregorian_v1_gen_e_timestamp(value_array_type *val_array, char *locale, event_type event);

/*
*  Routine:  cal_standard_gregorian_v1_gen_i_timestamp
*
*  Description:  Create an interval time-stamp from a parsed version of the 
*		 input string
*
*  Arguments: val_array1 -- (IN) : parsed representation of beginning of interval
*             val_array2 -- (IN) : parsed representation of ending of interval
*             locale   --      (IN) : current locale property value
*             interval --      (OUT): interval time-stamp
*
*  Return Value: Error code
*
*  Errors: conversion_error
*
*  Side Effects: Overwrites interval
*/

error_type cal_standard_gregorian_v1_gen_i_timestamp(value_array_type *val_array1, char *locale, interval_type interval);
#endif
