/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/fv/fv_tables.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
/*
* fv_tables.h -- internal data structure definitions (generated)
*/

#ifndef fv_tables
#define fv_tables

#define fv_max_field_tables 59
#define fv_max_field_functions 14

typedef struct {
    int min_index;
    int max_index;
    char *(*field_values)[];
} field_value_table;

extern field_value_table field_value_tables[];

typedef struct {
				/* int should be changed to unsigned long */
				/*            \				  */
    fv_error_type (*fv_table_index_to_string)(int, string_struct);
    fv_error_type (*fv_table_string_to_index)(char *, int *, int *);
				/*                     /		  */
				/* same here, change to unsigned long     */
} field_value_function;

extern field_value_function field_value_functions[];

extern char *table_names[];

#endif

/* american */

extern char *fv_ascii_american_english_ua_classday_term_names[];

extern fv_error_type fv_ascii_american_english_cardinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_american_english_cardinal_name_string_to_index(char*, int*, int*);
extern fv_error_type fv_ascii_american_english_ordinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_american_english_ordinal_name_string_to_index(char*, int*, int*);

/* arabic */

extern fv_error_type fv_ascii_arabic_cardinal_symbol_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_arabic_cardinal_symbol_string_to_index(char*, int*, int*);

/* british */

extern char *fv_ascii_british_english_ua_classday_term_names[];

extern fv_error_type fv_ascii_british_english_cardinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_british_english_cardinal_name_string_to_index(char*, int*, int*);
extern fv_error_type fv_ascii_british_english_ordinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_british_english_ordinal_name_string_to_index(char*, int*, int*);

/* english */

extern char *fv_ascii_english_gregorian_day_of_week_names[];
extern char *fv_ascii_english_gregorian_era_names[];
extern char *fv_ascii_english_gregorian_new_era_names[];
extern char *fv_ascii_english_gregorian_meridiem_names[];
extern char *fv_ascii_english_gregorian_month_names[];
extern char *fv_ascii_english_time_card_shift_names[];
extern char *fv_ascii_english_ua_classday_day_of_week_names[];
extern char *fv_ascii_english_ua_classday_meridiem_names[];

extern fv_error_type fv_ascii_english_ordinal_symbol_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_english_ordinal_symbol_string_to_index(char*, int*, int*);

/* chinese */

extern char *fv_gb_chinese_gregorian_day_of_week_names[];
extern char *fv_gb_chinese_gregorian_era_names[];
extern char *fv_gb_chinese_gregorian_meridiem_names[];
extern char *fv_gb_chinese_gregorian_month_names[];
extern char *fv_gb_chinese_time_card_shift_names[];
extern char *fv_gb_chinese_ua_classday_term_names[];
extern char *fv_gb_chinese_ua_classday_day_of_week_names[];
extern char *fv_gb_chinese_ua_classday_meridiem_names[];

extern fv_error_type fv_gb_chinese_cardinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_gb_chinese_cardinal_name_string_to_index(char*, int*, int*);
extern fv_error_type fv_gb_chinese_ordinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_gb_chinese_ordinal_name_string_to_index(char*, int*, int*);

extern char *fv_hz_chinese_gregorian_day_of_week_names[];
extern char *fv_hz_chinese_gregorian_era_names[];
extern char *fv_hz_chinese_gregorian_meridiem_names[];
extern char *fv_hz_chinese_gregorian_month_names[];
extern char *fv_hz_chinese_time_card_shift_names[];
extern char *fv_hz_chinese_ua_classday_term_names[];
extern char *fv_hz_chinese_ua_classday_day_of_week_names[];
extern char *fv_hz_chinese_ua_classday_meridiem_names[];

extern fv_error_type fv_hz_chinese_cardinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_hz_chinese_cardinal_name_string_to_index(char*, int*, int*);
extern fv_error_type fv_hz_chinese_ordinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_hz_chinese_ordinal_name_string_to_index(char*, int*, int*);

/* mandarin */

extern char *fv_ascii_romanized_mandarin_gregorian_day_of_week_names[];
extern char *fv_ascii_romanized_mandarin_gregorian_era_names[];
extern char *fv_ascii_romanized_mandarin_gregorian_meridiem_names[];
extern char *fv_ascii_romanized_mandarin_gregorian_month_names[];
extern char *fv_ascii_romanized_mandarin_time_card_shift_names[];
extern char *fv_ascii_romanized_mandarin_ua_classday_term_names[];
extern char *fv_ascii_romanized_mandarin_ua_classday_day_of_week_names[];
extern char *fv_ascii_romanized_mandarin_ua_classday_meridiem_names[];

extern fv_error_type fv_ascii_romanized_mandarin_cardinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_romanized_mandarin_cardinal_name_string_to_index(char*, int*, int*);
extern fv_error_type fv_ascii_romanized_mandarin_ordinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_romanized_mandarin_ordinal_name_string_to_index(char*, int*, int*);

/* japanese */

extern char *fv_ascii_romanized_japanese_gregorian_day_of_week_names[];
extern char *fv_ascii_romanized_japanese_gregorian_era_names[];
extern char *fv_ascii_romanized_japanese_gregorian_meridiem_names[];
extern char *fv_ascii_romanized_japanese_gregorian_month_names[];
extern char *fv_ascii_romanized_japanese_gregorian_day_of_month_names[];
extern char *fv_ascii_romanized_japanese_time_card_shift_names[];
extern char *fv_ascii_romanized_japanese_ua_classday_term_names[];
extern char *fv_ascii_romanized_japanese_ua_classday_day_of_week_names[];
extern char *fv_ascii_romanized_japanese_ua_classday_meridiem_names[];

extern fv_error_type fv_ascii_romanized_japanese_cardinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_romanized_japanese_cardinal_name_string_to_index(char*, int*, int*);
extern fv_error_type fv_ascii_romanized_japanese_ordinal_name_index_to_string(int, string_struct);
extern fv_error_type fv_ascii_romanized_japanese_ordinal_name_string_to_index(char*, int*, int*);

/* danish */

extern char *fv_latex_danish_gregorian_day_of_week_names[];
extern char *fv_latex_danish_gregorian_era_names[];
extern char *fv_latex_danish_gregorian_meridiem_names[];
extern char *fv_latex_danish_gregorian_month_names[];
extern char *fv_latex_danish_time_card_shift_names[];
extern char *fv_latex_danish_ua_classday_term_names[]; 
extern char *fv_latex_danish_ua_classday_day_of_week_names[];
extern char *fv_latex_danish_ua_classday_meridiem_names[];

/* german */

extern char *fv_latex_german_gregorian_day_of_week_names[];
extern char *fv_latex_german_gregorian_era_names[];
extern char *fv_latex_german_gregorian_meridiem_names[];
extern char *fv_latex_german_gregorian_month_names[];
extern char *fv_latex_german_time_card_shift_names[];
extern char *fv_latex_german_ua_classday_term_names[];
extern char *fv_latex_german_ua_classday_day_of_week_names[];
extern char *fv_latex_german_ua_classday_meridiem_names[];

